<?php

namespace Drupal\Tests\prometheusio_exporter_update\Unit\Plugin\MetricsCollector;

use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\prometheusio_exporter_update\Plugin\MetricsCollector\UpdateStatusCollector;
use Drupal\Tests\UnitTestCase;
use Drupal\update\UpdateManagerInterface;
use Drupal\Tests\prometheusio_exporter\Unit\Plugin\MetricsCollector\AbstractTestBaseMetrics;

/**
 * @coversDefaultClass \Drupal\prometheusio_exporter_update\Plugin\MetricsCollector\UpdateStatusCollector
 * @group prometheusio_exporter
 */
class UpdateStatusCollectorTest extends AbstractTestBaseMetrics {

  /**
   * @covers ::collectMetrics
   */
  public function testCollectMetrics() {

    $updateManager = $this->prophesize(UpdateManagerInterface::class);
    $updateManager->getProjects()->willReturn($this->getTestProjectData());

    $moduleHandler = $this->prophesize(ModuleHandlerInterface::class);
    $moduleHandler->moduleExists('update')->willReturn(TRUE);

    $collector = new UpdateStatusCollector(['description' => 'Dummy description.'], 'update_status_test', [], $this->prometheusBridge, $updateManager->reveal(), $moduleHandler->reveal());
    $collector->executeMetrics();

    $this->assertEquals(<<<EOD
# HELP drupal_update_status_test_core_version Drupal core version
# TYPE drupal_update_status_test_core_version gauge
drupal_update_status_test_core_version{version="8.6.1"} 1
# HELP drupal_update_status_test_module_version Drupal module version
# TYPE drupal_update_status_test_module_version gauge
drupal_update_status_test_module_version{name="test_module",version="8.x-1.3"} 1
# HELP drupal_update_status_test_theme_version Drupal theme version
# TYPE drupal_update_status_test_theme_version gauge
drupal_update_status_test_theme_version{name="test_theme",version="8.x-2.x"} 1

EOD, $this->prometheusBridge->render());
  }

  /**
   * Provides test data for the update manager.
   *
   * @return array
   *   The test project data.
   */
  protected function getTestProjectData() {

    return [
      'test_core' => [
        'project_type' => 'core',
        'info' => [
          'version' => '8.6.1',
        ],
      ],
      'test_module' => [
        'project_type' => 'module',
        'info' => [
          'version' => '8.x-1.3',
        ],
      ],
      'test_theme' => [
        'project_type' => 'theme',
        'info' => [
          'version' => '8.x-2.x',
        ],
      ],
    ];
  }

}
