<?php

namespace Drupal\prometheusio_exporter\Bridge;

use Drupal\Core\Site\Settings;
use Prometheus\CollectorRegistry;
use Prometheus\RenderTextFormat;
use Prometheus\Storage\Adapter;
use Prometheus\Storage\APC;
use Prometheus\Storage\InMemory;
use Prometheus\Storage\Redis;
use Drupal\prometheusio_exporter\MetricsCollectorManager;
use Drupal\prometheusio_exporter\Prometheus\Storage\DrupalCache;
use Symfony\Component\DependencyInjection\ContainerAwareTrait;

class PrometheusBridge implements PrometheusBridgeInterface {

  use ContainerAwareTrait;

  /**
   * The site settings.
   *
   * @var \Drupal\Core\Site\Settings
   */
  protected $settings;

  /**
   * @var \Prometheus\RenderTextFormat
   */
  protected $renderer;

  public function __construct(Settings $settings, Adapter $drupalCache){
    $storageType = $settings->get('prometheus_metrics_storage_type');
    switch($storageType){
      case 'redis':
        $options = [
          'host'=> $settings->get('prometheus_metrics_redis_host'),
          'port'=> $settings->get('prometheus_metrics_redis_port'),
          'timeout' => $settings->get('prometheus_metrics_redis_timeout'),
          'read_timeout' => $settings->get('prometheus_metrics_redis_read_timeout'),
          'persistent_connections' => $settings->get('prometheus_metrics_redis_persist_conns'),
          'password'=>null,
          'database' => $settings->get('prometheus_metrics_redis_database')
        ];
        $storageAdapter = new Redis($options);
        break;
      case 'apc':
        $storageAdapter = new APC();
        break;
      case 'inmemory':
        $storageAdapter = new InMemory();
        break;
      default:
        //default to DrupalCache storage adapter
        $storageAdapter = $drupalCache;
        break;
    }

    $this->collectorRegistry = new CollectorRegistry($storageAdapter, false);
    $this->renderer = new RenderTextFormat();
  }

  public function render(): string {
    // Now execute each metrics plugin so that their results can be put into storage.
    // i.e. in the collectorRegistry's storage.
    $this->container->get('prometheusio_exporter.metrics_collector_manager')->executeMetricsPlugins();
    return $this->renderer->render($this->collectorRegistry->getMetricFamilySamples());
  }

  public function getCounter(string $namespace, string $name, string $help, array $labels = []) {
    return $this->collectorRegistry->getOrRegisterCounter(
      $namespace,
      $name,
      $help,
      $labels
    );
  }

  public function getGauge(string $namespace, string $name, string $help, array $labels = []) {
    return $this->collectorRegistry->getOrRegisterGauge(
      $namespace,
      $name,
      $help,
      $labels
    );
  }

  public function getHistogram(string $namespace, string $name, string $help, array $labels = [], array $buckets = null) {
    return $this->collectorRegistry->getOrRegisterHistogram(
      $namespace,
      $name,
      $help,
      $labels,
      $buckets
    );
  }

}
