<?php

namespace Drupal\prometheusio_exporter\Plugin\MetricsCollector;

use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\prometheusio_exporter\Plugin\BaseMetricsCollector;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\prometheusio_exporter\Bridge\PrometheusBridgeInterface;

/**
 * Collects metrics for the total node count.
 *
 * @MetricsCollector(
 *   id = "user_count",
 *   title = @Translation("User count"),
 *   description = @Translation("Total user count.")
 * )
 */
class UserCount extends BaseMetricsCollector {

  /**
   * The user type storage.
   *
   * @var \Drupal\Core\Entity\EntityStorageInterface
   */
  protected $userStorage;

  /**
   * UserCount constructor.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Entity\EntityStorageInterface $user_storage
   *   The user type storage.
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    PrometheusBridgeInterface $promBridge,
    EntityStorageInterface $user_storage
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $promBridge);
    $this->userStorage = $user_storage;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('prometheusio_exporter.prometheus_bridge'),
      $container->get('entity_type.manager')->getStorage('user')
    );
  }

  /**
   * Gets a count for this metric.
   *
   * @param int $status
   *   (optional) The user status to filter by.
   *
   * @return int
   *   The user count.
   */
  protected function getCount(int $status = NULL) {
    $query = $this->userStorage->getQuery();
    $query->accessCheck(TRUE);
    if (!is_null($status)) {
      $query->condition('status', $status);
    }
    $count = $query->count()->execute();
    return $count;
  }

  /**
   * {@inheritdoc}
   */
  public function executeMetrics() {
    $namespace_name_help = [$this->getNamespace(), 'total', $this->getDescription()];
    $this->promBridge->getGauge(...$namespace_name_help)
                     ->set($this->getCount());
    $this->promBridge->getGauge(...$namespace_name_help, ...[['status']])
                     ->set($this->getCount(TRUE), ['active']);
    $this->promBridge->getGauge(...$namespace_name_help, ...[['status']])
                     ->set($this->getCount(FALSE), ['blocked']);
  }

}
