<?php

namespace Drupal\prometheusio_exporter\Prometheus\Storage;

use Prometheus\Storage\InMemory;
use Drupal\Core\Cache\CacheBackendInterface;

class DrupalCache extends InMemory {

  /**
   * @var CacheBackendInterface.
   */
  protected $cacheBackend;

  public function __construct(CacheBackendInterface $cacheBackend) {
    $this->cacheBackend = $cacheBackend;
  }

  /**
   * {@inheritdoc}
   */
  public function collect(): array {
    foreach (['counters', 'gauges', 'histograms'] as $type) {
      $this->{$type} = $this->cacheBackend->get('prometheusio_exporter:' . $type)->data ?? [];
    }
    return parent::collect();
  }

  /**
   * {@inheritdoc}
   */
  public function updateHistogram(array $data): void {
    $this->commonUpdateMetric('histograms', __FUNCTION__, $data);
  }

  /**
   * {@inheritdoc}
   */
  public function updateGauge(array $data): void {
    $this->commonUpdateMetric('gauges', __FUNCTION__, $data);
  }

  /**
   * {@inheritdoc}
   */
  public function updateCounter(array $data): void {
    $this->commonUpdateMetric('counters', __FUNCTION__, $data);
  }

  protected function commonUpdateMetric(string $type, string $method, array $data) {
    parent::$method($data);
    $data = array_merge($this->cacheBackend->get('prometheusio_exporter:' . $type)->data ?? [], $this->{$type});
    $this->cacheBackend->set('prometheusio_exporter:' . $type, $data);
  }

  /**
   * {@inheritdoc}
   */
  public function wipeStorage(): void {
    parent::wipeStorage();
    $this->cacheBackend->deleteAll();
  }

}
