<?php

namespace Drupal\Tests\prometheusio_exporter\Unit\Plugin\MetricsCollector;

use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Entity\Query\QueryInterface;
use Drupal\prometheusio_exporter\Plugin\MetricsCollector\UserCount;
use Drupal\Tests\UnitTestCase;

/**
 * @coversDefaultClass \Drupal\prometheusio_exporter\Plugin\MetricsCollector\UserCount
 * @group prometheusio_exporter
 */
class UserCountTest extends AbstractTestBaseMetrics {

  /**
   * @covers ::collectMetrics
   */
  public function testCollectMetrics() {
    $total = 12;
    $blocked = 5;
    $activated = $total - $blocked;
    $userQuery1 = $this->prophesize(QueryInterface::class);
    $userQuery1->accessCheck(TRUE)->willReturn($userQuery1);
    $userQuery1->count()->willReturn($userQuery1);
    $userQuery1->execute()->willReturn($total);

    $userQuery2 = $this->prophesize(QueryInterface::class);
    $userQuery2->accessCheck(TRUE)->willReturn($userQuery2);
    $userQuery2->condition('status', 1)->willReturn($userQuery2);
    $userQuery2->count()->willReturn($userQuery2);
    $userQuery2->execute()->willReturn($activated);

    $userQuery3 = $this->prophesize(QueryInterface::class);
    $userQuery3->accessCheck(TRUE)->willReturn($userQuery3);
    $userQuery3->condition('status', 0)->willReturn($userQuery3);
    $userQuery3->count()->willReturn($userQuery3);
    $userQuery3->execute()->willReturn($blocked);

    $userTypeStorage = $this->prophesize(EntityStorageInterface::class);
    $userTypeStorage->getQuery()
      ->willReturn($userQuery1, $userQuery2, $userQuery3);

    $configuration = [
      'enabled' => TRUE,
      'weight' => 0,
      'settings' => [],
    ];
    $definition = [
      'provider' => 'user_count',
      'description' => 'Test description',
    ];

    $collector = new UserCount($configuration, 'user_count', $definition, $this->prometheusBridge,
      $userTypeStorage->reveal());

    $collector->executeMetrics();

    $this->assertEquals(<<<EOD
# HELP drupal_user_count_total Test description
# TYPE drupal_user_count_total gauge
drupal_user_count_total 12
drupal_user_count_total{status="active"} 7
drupal_user_count_total{status="blocked"} 5

EOD, $this->prometheusBridge->render());
  }

}
